@@ -53,7 +53,14 @@ REQUEST_HEADERS = {'Accept': 'application/bitcoin-paymentrequest', 'User-Agent':
 ACK_HEADERS = {'Content-Type':'application/bitcoin-payment','Accept':'application/bitcoin-paymentack','User-Agent':'Electrum'}
 
 ca_path = requests.certs.where()
-ca_list, ca_keyID = x509.load_certificates(ca_path)
+ca_list = None
+ca_keyID = None
+
+def load_ca_list():
+    global ca_list, ca_keyID
+    if ca_list is None:
+        ca_list, ca_keyID = x509.load_certificates(ca_path)
+
 
 
 # status of payment requests
@@ -155,6 +162,7 @@ class PaymentRequest:
             return False
 
     def verify_x509(self, paymntreq):
+        load_ca_list()
         if not ca_list:
             self.error = "Trusted certificate authorities list not found"
             return False
@@ -334,6 +342,7 @@ def sign_request_with_alias(pr, alias, alias_privkey):
 
 def verify_cert_chain(chain):
     """ Verify a chain of certificates. The last certificate is the CA"""
+    load_ca_list()
     # parse the chain
     cert_num = len(chain)
     x509_chain = []
@@ -448,18 +457,13 @@ def make_request(config, req):
 
 class InvoiceStore(object):
 
-    def __init__(self, config):
-        self.config = config
+    def __init__(self, storage):
+        self.storage = storage
         self.invoices = {}
-        self.load_invoices()
+        d = self.storage.get('invoices', {})
+        self.load(d)
 
-    def load_invoices(self):
-        path = os.path.join(self.config.path, 'invoices')
-        try:
-            with open(path, 'r') as f:
-                d = json.loads(f.read())
-        except:
-            return
+    def load(self, d):
         for k, v in d.items():
             try:
                 pr = PaymentRequest(v.get('hex').decode('hex'))
@@ -469,6 +473,15 @@ class InvoiceStore(object):
             except:
                 continue
 
+    def import_file(self, path):
+        try:
+            with open(path, 'r') as f:
+                d = json.loads(f.read())
+                self.load(d)
+        except:
+            return
+        self.save()
+
     def save(self):
         l = {}
         for k, pr in self.invoices.items():
@@ -477,10 +490,7 @@ class InvoiceStore(object):
                 'requestor': pr.requestor,
                 'txid': pr.tx
             }
-        path = os.path.join(self.config.path, 'invoices')
-        with open(path, 'w') as f:
-            s = json.dumps(l, indent=4, sort_keys=True)
-            r = f.write(s)
+        self.storage.put('invoices', l)
 
     def get_status(self, key):
         pr = self.get(key)
