@@ -220,6 +220,7 @@ class Abstract_Wallet(PrintError):
 
     def load_addresses(self):
         d = self.storage.get('addresses', {})
+        if type(d) != dict: d={}
         self.receiving_addresses = d.get('receiving', [])
         self.change_addresses = d.get('change', [])
 
@@ -1584,6 +1585,8 @@ class Simple_Deterministic_Wallet(Deterministic_Wallet, Simple_Wallet):
         self.keystore.check_password(password)
 
     def update_password(self, old_pw, new_pw, encrypt=False):
+        if old_pw is None and self.has_password():
+            raise InvalidPassword()
         self.keystore.update_password(old_pw, new_pw)
         self.save_keystore()
         self.storage.set_password(new_pw, encrypt)
@@ -1688,11 +1691,14 @@ class Multisig_Wallet(Deterministic_Wallet, P2SH):
         return [self.keystores[i] for i in sorted(self.keystores.keys())]
 
     def update_password(self, old_pw, new_pw, encrypt=False):
+        if old_pw is None and self.has_password():
+            raise InvalidPassword()
         for name, keystore in self.keystores.items():
             if keystore.can_change_password():
                 keystore.update_password(old_pw, new_pw)
                 self.storage.put(name, keystore.dump())
         self.storage.set_password(new_pw, encrypt)
+        self.storage.write()
 
     def check_password(self, password):
         self.keystore.check_password(password)
