@@ -6,7 +6,7 @@ from binascii import hexlify, unhexlify
 from functools import partial
 
 from electrum.bitcoin import (bc_address_to_hash_160, xpub_from_pubkey,
-                              public_key_to_bc_address, EncodeBase58Check,
+                              public_key_to_p2pkh, EncodeBase58Check,
                               TYPE_ADDRESS, TYPE_SCRIPT)
 from electrum.i18n import _
 from electrum.plugins import BasePlugin, hook
@@ -27,11 +27,10 @@ class TrezorCompatibleKeyStore(Hardware_KeyStore):
     def get_client(self, force_pair=True):
         return self.plugin.get_client(self, force_pair)
 
-    def decrypt_message(self, pubkey, message, password):
+    def decrypt_message(self, sequence, message, password):
         raise RuntimeError(_('Electrum and %s encryption and decryption are currently incompatible') % self.device)
-        address = public_key_to_bc_address(pubkey.decode('hex'))
         client = self.get_client()
-        address_path = self.address_id(address)
+        address_path = self.get_derivation() + "/%d/%d"%sequence
         address_n = client.expand_path(address_path)
         payload = base64.b64decode(message)
         nonce, message, msg_hmac = payload[:33], payload[33:-8], payload[-8:]
